@@ -1,6 +1,6 @@
 /* Read a symbol table in MIPS' format (Third-Eye).
 
-   Copyright (C) 1986-2023 Free Software Foundation, Inc.
+   Copyright (C) 1986-2025 Free Software Foundation, Inc.
 
    Contributed by Alessandro Forin (af@cs.cmu.edu) at CMU.  Major work
    by Per Bothner, John Gilmore and Ian Lance Taylor at Cygnus Support.
@@ -23,7 +23,6 @@
 /* Read symbols from an ECOFF file.  Most of the work is done in
    mdebugread.c.  */
 
-#include "defs.h"
 #include "bfd.h"
 #include "symtab.h"
 #include "objfiles.h"
@@ -33,8 +32,8 @@
 #include "coff/sym.h"
 #include "coff/internal.h"
 #include "coff/ecoff.h"
-#include "libcoff.h"		/* Private BFD COFF information.  */
-#include "libecoff.h"		/* Private BFD ECOFF information.  */
+#include "libcoff.h"
+#include "libecoff.h"
 #include "elf/common.h"
 #include "elf/internal.h"
 #include "elf/mips.h"
@@ -67,20 +66,16 @@ mipscoff_symfile_read (struct objfile *objfile, symfile_add_flags symfile_flags)
 {
   bfd *abfd = objfile->obfd.get ();
 
-  minimal_symbol_reader reader (objfile);
-
   /* Now that the executable file is positioned at symbol table,
      process it and define symbols accordingly.  */
 
-  if (!((*ecoff_backend (abfd)->debug_swap.read_debug_info)
-	(abfd, NULL, &ecoff_data (abfd)->debug_info)))
-    error (_("Error reading symbol table: %s"), bfd_errmsg (bfd_get_error ()));
-
-  mdebug_build_psymtabs (reader, objfile, &ecoff_backend (abfd)->debug_swap,
-			 &ecoff_data (abfd)->debug_info);
+  mipsmdebug_build_psymtabs (objfile, &ecoff_backend (abfd)->debug_swap,
+			     &ecoff_data (abfd)->debug_info);
 
   /* Add alpha coff dynamic symbols.  */
 
+  minimal_symbol_reader reader (objfile);
+
   read_alphacoff_dynamic_symtab (reader, objfile);
 
   /* Install any minimal symbols that have been collected as the current
@@ -375,9 +370,7 @@ static const struct sym_fns ecoff_sym_fns =
   NULL,				/* sym_probe_fns */
 };
 
-void _initialize_mipsread ();
-void
-_initialize_mipsread ()
+INIT_GDB_FILE (mipsread)
 {
   add_symtab_fns (bfd_target_ecoff_flavour, &ecoff_sym_fns);
 }
